@@ -66,6 +66,11 @@ class NapariSTPT:
 
         self.crop = False
         self.old_method = True
+
+        self.start_slice = None
+        self.end_slice = None
+
+        self.image_translation = None
         
     def CropOriginal(self, volume):
 
@@ -228,8 +233,19 @@ class NapariSTPT:
         with napari.gui_qt():
             if self.cb_C1.isChecked():
                 print("loading C1")
-                volume_1_temp = (ds.sel(channel=1, type='mosaic', z=0).to_array(
-                ).data * bscale + bzero).astype(dtype=np.float32)
+                volume_1_temp = (ds.sel(channel=1, type='mosaic', z=0).to_array().data * bscale + bzero).astype(dtype=np.float32)
+
+                if self.start_slice.text() == "":
+                    start_slice_number = 0
+                else:
+                    start_slice_number = int(self.start_slice.text())
+
+                if self.end_slice.text() == "":
+                    end_slice_number = volume_1_temp.shape[0]-1
+                else:
+                    end_slice_number = int(self.end_slice.text())
+
+                number_of_slices = end_slice_number - start_slice_number + 1
 
                 if self.crop:
                     size_y = int(math.floor(
@@ -246,26 +262,22 @@ class NapariSTPT:
                     start_y = 0
                     start_z = 0
 
-                volume_1 = np.zeros(
-                    (self.optical_slices*volume_1_temp.shape[0], size_y, size_z), dtype=np.float32)
-                volume_1[0::self.optical_slices, :, :] = volume_1_temp[:,
-                                                                       start_y:start_y+size_y, start_z:start_z+size_z]
+                volume_1 = np.zeros((self.optical_slices*number_of_slices, size_y, size_z), dtype=np.float32)
+                volume_1[0::self.optical_slices, :, :] = volume_1_temp[start_slice_number:end_slice_number+1, start_y:start_y+size_y, start_z:start_z+size_z]
 
                 for optical_slice in range(1, self.optical_slices):
                     print(f"loading optical slice {optical_slice}")
-                    volume_1_temp = (ds.sel(channel=1, type='mosaic', z=optical_slice).to_array(
-                    ).data * bscale + bzero).astype(dtype=np.float32)
-                    volume_1[optical_slice::self.optical_slices, :, :] = volume_1_temp[:,
-                                                                                       start_y:start_y+size_y, start_z:start_z+size_z]
+                    volume_1_temp = (ds.sel(channel=1, type='mosaic', z=optical_slice).to_array().data * bscale + bzero).astype(dtype=np.float32)
+                    volume_1[optical_slice::self.optical_slices, :, :] = volume_1_temp[start_slice_number:end_slice_number+1,start_y:start_y+size_y, start_z:start_z+size_z]
 
-                # self.aligned_1 = volume_1
+                self.aligned_1 = volume_1
 
                 print("aligning C1")
-                self.aligned_1 = self.Align(volume_1, resolution, output_resolution)
+                self.aligned_1 = self.Align(volume_1, resolution, output_resolution, start_slice_number)
                 
                 self.shape = self.aligned_1.shape
                 self.viewer.add_image([self.aligned_1], name='C1', scale=(
-                    15/self.optical_slices, output_resolution, output_resolution), blending='additive', colormap='bop purple', translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+                    15/self.optical_slices, output_resolution, output_resolution), blending='additive', colormap='bop purple')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
 
             if self.cb_C2.isChecked():
                 print("loading C2")
@@ -273,6 +285,18 @@ class NapariSTPT:
                 volume_2_temp = (ds.sel(channel=2, type='mosaic', z=0).to_array(
                 ).data * bscale + bzero).astype(dtype=np.float32)
 
+                if self.start_slice.text() == "":
+                    start_slice_number = 0
+                else:
+                    start_slice_number = int(self.start_slice.text())
+
+                if self.end_slice.text() == "":
+                    end_slice_number = volume_2_temp.shape[0]-1
+                else:
+                    end_slice_number = int(self.end_slice.text())
+
+                number_of_slices = end_slice_number - start_slice_number + 1
+
                 if self.crop:
                     size_y = int(math.floor(
                         volume_2_temp.shape[1]/self.crop_size_ratio_x))
@@ -289,31 +313,43 @@ class NapariSTPT:
                     start_z = 0
 
                 volume_2 = np.zeros(
-                    (self.optical_slices*volume_2_temp.shape[0], size_y, size_z), dtype=np.float32)
-                volume_2[0::self.optical_slices, :, :] = volume_2_temp[:,
+                    (self.optical_slices*number_of_slices, size_y, size_z), dtype=np.float32)
+                volume_2[0::self.optical_slices, :, :] = volume_2_temp[start_slice_number:end_slice_number+1,
                                                                        start_y:start_y+size_y, start_z:start_z+size_z]
 
                 for optical_slice in range(1, self.optical_slices):
                     print(f"loading optical slice {optical_slice}")
                     volume_2_temp = (ds.sel(channel=2, type='mosaic', z=optical_slice).to_array(
                     ).data * bscale + bzero).astype(dtype=np.float32)
-                    volume_2[optical_slice::self.optical_slices, :, :] = volume_2_temp[:,
+                    volume_2[optical_slice::self.optical_slices, :, :] = volume_2_temp[start_slice_number:end_slice_number+1,
                                                                                        start_y:start_y+size_y, start_z:start_z+size_z]
 
                 # self.aligned_2 = volume_2
 
                 print("aligning C2")
                 self.aligned_2 = self.Align(
-                    volume_2, resolution, output_resolution)
+                    volume_2, resolution, output_resolution, start_slice_number)
                 self.shape = self.aligned_2.shape
                 self.viewer.add_image([self.aligned_2], name='C2', scale=(
-                    15.0/self.optical_slices, output_resolution, output_resolution), blending='additive', colormap='red', translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+                    15.0/self.optical_slices, output_resolution, output_resolution), blending='additive', colormap='red')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
 
             if self.cb_C3.isChecked():
                 print("loading C3")
                 volume_3_temp = (ds.sel(channel=3, type='mosaic', z=0).to_array(
                 ).data * bscale + bzero).astype(dtype=np.float32)
 
+                if self.start_slice.text() == "":
+                    start_slice_number = 0
+                else:
+                    start_slice_number = int(self.start_slice.text())
+
+                if self.end_slice.text() == "":
+                    end_slice_number = volume_3_temp.shape[0]-1
+                else:
+                    end_slice_number = int(self.end_slice.text())
+
+                number_of_slices = end_slice_number - start_slice_number + 1
+
                 if self.crop:
                     size_y = int(math.floor(
                         volume_3_temp.shape[1]/self.crop_size_ratio_x))
@@ -330,31 +366,43 @@ class NapariSTPT:
                     start_z = 0
 
                 volume_3 = np.zeros(
-                    (self.optical_slices*volume_3_temp.shape[0], size_y, size_z), dtype=np.float32)
-                volume_3[0::self.optical_slices, :, :] = volume_3_temp[:,
+                    (self.optical_slices*number_of_slices, size_y, size_z), dtype=np.float32)
+                volume_3[0::self.optical_slices, :, :] = volume_3_temp[start_slice_number:end_slice_number+1,
                                                                        start_y:start_y+size_y, start_z:start_z+size_z]
 
                 for optical_slice in range(1, self.optical_slices):
                     print(f"loading optical slice {optical_slice}")
                     volume_3_temp = (ds.sel(channel=3, type='mosaic', z=optical_slice).to_array(
                     ).data * bscale + bzero).astype(dtype=np.float32)
-                    volume_3[optical_slice::self.optical_slices, :, :] = volume_3_temp[:,
+                    volume_3[optical_slice::self.optical_slices, :, :] = volume_3_temp[start_slice_number:end_slice_number+1,
                                                                                        start_y:start_y+size_y, start_z:start_z+size_z]
 
                 # self.aligned_3 = volume_3
 
                 print("aligning C3")
                 self.aligned_3 = self.Align(
-                    volume_3, resolution, output_resolution)
+                    volume_3, resolution, output_resolution, start_slice_number)
                 self.shape = self.aligned_3.shape
                 self.viewer.add_image([self.aligned_3], name='C3', scale=(
-                    15/self.optical_slices, output_resolution, output_resolution), blending='additive', colormap='green', translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+                    15/self.optical_slices, output_resolution, output_resolution), blending='additive', colormap='green')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
 
             if self.cb_C4.isChecked():
                 print("loading C4")
                 volume_4_temp = (ds.sel(channel=4, type='mosaic', z=0).to_array(
                 ).data * bscale + bzero).astype(dtype=np.float32)
 
+                if self.start_slice.text() == "":
+                    start_slice_number = 0
+                else:
+                    start_slice_number = int(self.start_slice.text())
+
+                if self.end_slice.text() == "":
+                    end_slice_number = volume_4_temp.shape[0]-1
+                else:
+                    end_slice_number = int(self.end_slice.text())
+
+                number_of_slices = end_slice_number - start_slice_number + 1
+
                 if self.crop:
                     size_y = int(math.floor(
                         volume_4_temp.shape[1]/self.crop_size_ratio_x))
@@ -371,25 +419,25 @@ class NapariSTPT:
                     start_z = 0
 
                 volume_4 = np.zeros(
-                    (self.optical_slices*volume_4_temp.shape[0], size_y, size_z), dtype=np.float32)
-                volume_4[0::self.optical_slices, :, :] = volume_4_temp[:,
+                    (self.optical_slices*number_of_slices, size_y, size_z), dtype=np.float32)
+                volume_4[0::self.optical_slices, :, :] = volume_4_temp[start_slice_number:end_slice_number+1,
                                                                        start_y:start_y+size_y, start_z:start_z+size_z]
 
                 for optical_slice in range(1, self.optical_slices):
                     print(f"loading optical slice {optical_slice}")
                     volume_4_temp = (ds.sel(channel=4, type='mosaic', z=optical_slice).to_array(
                     ).data * bscale + bzero).astype(dtype=np.float32)
-                    volume_4[optical_slice::self.optical_slices, :, :] = volume_4_temp[:,
+                    volume_4[optical_slice::self.optical_slices, :, :] = volume_4_temp[start_slice_number:end_slice_number+1,
                                                                                        start_y:start_y+size_y, start_z:start_z+size_z]
 
                 # self.aligned_4 = volume_4
 
                 print("aligning C4")
                 self.aligned_4 = self.Align(
-                    volume_4, resolution, output_resolution)
+                    volume_4, resolution, output_resolution, start_slice_number)
                 self.shape = self.aligned_4.shape
                 self.viewer.add_image([self.aligned_4], name='C4', scale=(
-                    15/self.optical_slices, output_resolution, output_resolution), blending='additive', colormap='bop blue', translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+                    15/self.optical_slices, output_resolution, output_resolution), blending='additive', colormap='bop blue')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
 
             # create naparimovie object
             #if self.movie is None:
@@ -557,17 +605,15 @@ class NapariSTPT:
                                       name='C4', blending='additive', colormap='bop blue')
 
 
-    def Align(self, volume, resolution, output_resolution):
+    def Align(self, volume, resolution, output_resolution, start_slice_number):
 
         spacing = (self.ds1['S001'].attrs['scale'])
         # size_multiplier = (resolution*0.1*spacing[0])/7.5
         # print(resolution*0.1*spacing[0])
         size_multiplier = (resolution*0.1*spacing[0])/output_resolution
-        size = (volume.shape[0], int(
-            size_multiplier*volume.shape[1]), int(size_multiplier*volume.shape[2]))
+        size = (volume.shape[0], int(size_multiplier*volume.shape[1]), int(size_multiplier*volume.shape[2]))
         aligned = np.zeros(size, dtype=np.float32)
-        size2D = (
-            int(size_multiplier*volume.shape[2]), int(size_multiplier*volume.shape[1]))
+        size2D = (int(size_multiplier*volume.shape[2]), int(size_multiplier*volume.shape[1]))
 
         print("spacing: {}".format(spacing))
         print("volume.shape[0]: {}".format(volume.shape[0]))
@@ -583,13 +629,12 @@ class NapariSTPT:
             #    slice_name = 'S0'+str(z+1)
             # if(z+1 > 99):
             #    slice_name = 'S'+str(z+1)
-            slice_name = self.slice_names[z]
+            slice_name = self.slice_names[z+start_slice_number]
 
             spacing = (self.ds1[slice_name].attrs['scale'])
             print(f"spacing {spacing}")
             # fixed.SetSpacing([1.6*spacing[0],1.6*spacing[1]])
-            fixed.SetSpacing([resolution*0.1*spacing[1],
-                             resolution*0.1*spacing[0]])
+            fixed.SetSpacing([resolution*0.1*spacing[1],resolution*0.1*spacing[0]])
 
             transform = sitk.Euler2DTransform()
 
@@ -606,10 +651,8 @@ class NapariSTPT:
             # transform.SetTranslation([-align_y[z]*0.1*spacing[1],-align_x[z]*0.1*spacing[0]])
             # transform.SetTranslation([0.5*-align_y[z],0.5*-align_x[z]])
             # transform.SetTranslation([0,0])
-            print('{}/{}, translate_x: {}, translate_y: {}'.format(z,
-                  z_size, alignX, alignY))
-            print('{}/{}, align_x[z]: {}, align_y[z]: {}'.format(z,
-                  z_size, self.align_x[z], self.align_y[z]))
+            print('{}/{}, translate_x: {}, translate_y: {}'.format(z,z_size, alignX, alignY))
+            print('{}/{}, align_x[z]: {}, align_y[z]: {}'.format(z,z_size, self.align_x[z], self.align_y[z]))
 
             resampler = sitk.ResampleImageFilter()
             # resampler.SetReferenceImage(fixed)
@@ -1029,26 +1072,30 @@ class NapariSTPT:
             self.aligned_1 = self.aligned_1[:, int(
                 minX):int(maxX), int(minY):int(maxY)]
             self.viewer.layers.remove('C1')
+            self.image_translation = (int(minX*output_resolution), int(minY*output_resolution))
             self.viewer.add_image([self.aligned_1], name='C1', scale=(
-                15, output_resolution, output_resolution), blending='additive', colormap='bop purple', translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
+                15, output_resolution, output_resolution), blending='additive', colormap='bop purple')#, translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
         if self.aligned_2 is not None and self.cb_C2.isChecked() and any(i.name == 'C2' for i in self.viewer.layers):
             self.aligned_2 = self.aligned_2[:, int(
                 minX):int(maxX), int(minY):int(maxY)]
             self.viewer.layers.remove('C2')
+            self.image_translation = (int(minX*output_resolution), int(minY*output_resolution))
             self.viewer.add_image([self.aligned_2], name='C2', scale=(
-                15, output_resolution, output_resolution), blending='additive', colormap='red', translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
+                15, output_resolution, output_resolution), blending='additive', colormap='red')#, translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
         if self.aligned_3 is not None and self.cb_C3.isChecked() and any(i.name == 'C3' for i in self.viewer.layers):
             self.aligned_3 = self.aligned_3[:, int(
                 minX):int(maxX), int(minY):int(maxY)]
             self.viewer.layers.remove('C3')
+            self.image_translation = (int(minX*output_resolution), int(minY*output_resolution))
             self.viewer.add_image([self.aligned_3], name='C3', scale=(
-                15, output_resolution, output_resolution), blending='additive', colormap='green', translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
+                15, output_resolution, output_resolution), blending='additive', colormap='green')#, translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
         if self.aligned_4 is not None and self.cb_C4.isChecked() and any(i.name == 'C4' for i in self.viewer.layers):
             self.aligned_4 = self.aligned_4[:, int(
                 minX):int(maxX), int(minY):int(maxY)]
             self.viewer.layers.remove('C4')
+            self.image_translation = (int(minX*output_resolution), int(minY*output_resolution))
             self.viewer.add_image([self.aligned_4], name='C4', scale=(
-                15, output_resolution, output_resolution), blending='additive', colormap='bop blue', translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
+                15, output_resolution, output_resolution), blending='additive', colormap='bop blue')#, translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
             
             
         self.viewer.layers.remove('Shapes')
@@ -1150,7 +1197,8 @@ class NapariSTPT:
 
 
         new_shape = np.array(new_shape)
-        shapes_layer = self.viewer.add_shapes(new_shape, shape_type='polygon', name = "Shapes", scale=(15, 15, 15),)
+        output_resolution = float(self.pixel_size.text())
+        shapes_layer = self.viewer.add_shapes(new_shape, shape_type='polygon', name = "Shapes", scale=(15, output_resolution, output_resolution),)
     
 
 
@@ -1232,8 +1280,8 @@ class NapariSTPT:
                     weight2 = (z_level - x1) / (x2 - x1)
                     weight = 1 - weight2
 
-                    y = (weight * y1) + (weight2 * y2)
-                    z = (weight * z1) + (weight2 * z2)
+                    y = (weight * y1) + (weight2 * y2)# + (self.image_translation[0]/15)
+                    z = (weight * z1) + (weight2 * z2)#  + (self.image_translation[1]/15)
                     polygon_values.append((y, z))
 
 
@@ -1631,6 +1679,22 @@ class NapariSTPT:
             hbox.addStretch(1)
             vbox.addLayout(hbox)
 
+            
+
+            hbox = QtWidgets.QHBoxLayout()
+            hbox.addWidget(QtWidgets.QLabel(
+                "Slices range:"))
+            self.start_slice = QtWidgets.QLineEdit("")
+            self.start_slice.setMaximumWidth(50)
+            hbox.addWidget(self.start_slice)
+            hbox.addWidget(QtWidgets.QLabel(
+                "to"))
+            self.end_slice = QtWidgets.QLineEdit("")
+            self.end_slice.setMaximumWidth(50)
+            hbox.addWidget(self.end_slice)
+            hbox.addStretch(1)
+            vbox.addLayout(hbox)
+
             hbox = QtWidgets.QHBoxLayout()
             hbox.addWidget(QtWidgets.QLabel(
                 "Maximum number of optical slices:"))
